////////////////////////////////////////////////////
// Poisson blur stencil shader
// Copyright (c) 2008 oP group. All rights reserved.
////////////////////////////////////////////////////
#include <poisson>

//Tweakables
static const float fDist = 4.0;   // blur factor

texture StencilMap; // black/white stencil buffer image
sampler smpStencil = sampler_state { texture = <StencilMap>; }; 

float4 vecViewPort; // contains viewport pixel size in zw components

#define NUM_TAPS 12
static const float2 fTaps_Poisson[NUM_TAPS] = {
	{-.326,-.406},
	{-.840,-.074},
	{-.696, .457},
	{-.203, .621},
	{ .962,-.195},
	{ .473,-.480},
	{ .519, .767},
	{ .185,-.893},
	{ .507, .064},
	{ .896, .412},
	{-.322,-.933},
	{-.792,-.598}};

float4 blurPoissonPS( float2 Tex : TEXCOORD0) : COLOR0 
{
   float4 Stencil = 0;
   for (int i=0; i < NUM_TAPS; i++)
     Stencil += tex2D(smpStencil,Tex.xy + fTaps_Poisson[i]*fDist*vecViewPort.zw);
   return Stencil * (1.5/NUM_TAPS);
}

technique blur
{ 
   pass one
   { 
     PixelShader = compile ps_2_0 blurPoissonPS(); 
   }
} 

technique fallback { pass one { } }
