#include <transform>
#include <fog>
#include <pos>
#include <light>

float4 vecLight;

texture entSkin1;	// texture
sampler sBaseTex = sampler_state { Texture = <entSkin1>; MipFilter = Linear;	};

float4 vecSkill41;
float4 vecTime;

struct vsOut
{
	float4 Pos:    	POSITION;
	float  Fog:		FOG;		
	float4 Ambient:	COLOR;
	float2 Tex:   	TEXCOORD0;
};

vsOut twa_VS(
in float4 inPos: 	POSITION, 
in float3 inNormal:	NORMAL,
in float2 inTex: 	TEXCOORD0)
{
	vsOut Out;

	Out.Tex	= inTex;
	Out.Fog	= DoFog(inPos);
	
	float4 P = DoPos(inPos);
	
	float force = vecSkill41.x/25. * (25./300.);
	float direction = vecSkill41.y/100. * 2.*3.14;
	float oscillation = sin(vecTime.w * (vecSkill41.z/1000.));	
	if (oscillation<0)	oscillation *= 0.33;
	
	if ( P.y > 0 )
	{
		P.x += oscillation * (force*P.y) * sin(direction);
		P.z += oscillation * (force*P.y) * cos(direction);
		P.y -= abs(oscillation)*pow(force,2)*P.y;
	}
	Out.Pos = DoTransform(P);
   
	float facAmbient = vecSkill41.w / 50.;
	Out.Ambient = facAmbient*vecLight;
   	
	return Out;		
}


float4 twa_PS(vsOut In): COLOR
{
	float2 newTexCoord = In.Tex;
	
	float direction = vecSkill41.y/100. * 2.*3.14;
	float oscillation = sin(vecTime.w * (vecSkill41.z/500.));
	
	if ( newTexCoord.x > 0.256 && newTexCoord.x<0.995 && newTexCoord.y > 0.005 && newTexCoord.y < 0.995 )
	{
		newTexCoord.x += oscillation*0.005*sin(direction);
		newTexCoord.y += oscillation*0.005*cos(direction);
	}
	
	float4 base = tex2D(sBaseTex,newTexCoord);
	
	float diffuse;
	if (In.Tex.x < 0.25)	diffuse = 2. * (base.x+base.y+base.z) / 3.;
		else					diffuse = 1.;
	
	return float4(base.xyz*diffuse*In.Ambient,base.w);
}

/*technique twa
{
	pass One
	{		
		VertexShader = compile vs_1_1 twa_VS();
		PixelShader = compile ps_2_0 twa_PS();
	}
}*/

technique twaVSonly
{
	pass One
	{		
		VertexShader = compile vs_1_1 twa_VS();
	}
}

technique fallback { pass one { } }