#include <lights>
#include <fog>

texture mtlSkin1;
texture mtlSkin2;
texture mtlSkin3;
texture entSkin1;

sampler tSkin1 = sampler_state {Texture = <mtlSkin1>;};
sampler tSkin2 = sampler_state {Texture = <mtlSkin2>;};
sampler tFogBlend = sampler_state {Texture = <mtlSkin3>;};
sampler tBlend = sampler_state {Texture = <entSkin1>;};

float4x4 matWorldViewProj;
float4 ambientLighting = {0,0,0,0};
float4 vecTime;
float4 fogColor = {0.4,0.4,0.9,1};

////////////////////// - PASS ONE CODE - //////////////////////
struct VS_OUTPUT
{
	float4 Pos :	POSITION;
	float2 Tex0 :	TEXCOORD0;
	float2 Tex1 :	TEXCOORD1;
	float2 Tex2 :	TEXCOORD2;
	float2 Tex3 :	TEXCOORD3;
	float  Fog :	FOG;
	float4 Diffuse:	COLOR0;
	float4 Ambient:	COLOR1;
};

struct PS_INPUT
{
	float2 Tex0 :	TEXCOORD0;
	float2 Tex1 :	TEXCOORD1;
	float2 Tex2 :	TEXCOORD2;
	float2 Tex3 :	TEXCOORD2;
	float4 Diffuse:	COLOR0;
	float4 Ambient:	COLOR1;
};

VS_OUTPUT vs_main(VS_OUTPUT _in,float3 inNormal: NORMAL)
{
	_in.Fog = DoFog(_in.Pos);
	_in.Pos = mul(_in.Pos,matWorldViewProj);
	_in.Tex2 = _in.Tex0;
	_in.Tex3 = _in.Tex0;
	
	_in.Tex0 *= 16;
	_in.Tex1 = _in.Tex0;
	_in.Ambient = ambientLighting;
	
	float4 LightColor = 0;
	for(int i = 0;i < 8;i++)
		LightColor += DoLight(_in.Pos,inNormal.xyz,i);
	
	_in.Diffuse = LightColor;
	
	return _in;
}

VS_OUTPUT vs_main_FALLBACK(VS_OUTPUT _in,float3 inNormal: NORMAL)
{
	_in.Fog = DoFog(_in.Pos);
	_in.Pos = mul(_in.Pos,matWorldViewProj);
	_in.Tex2 = _in.Tex0;
	_in.Tex3 = _in.Tex0;
	
	_in.Tex0 *= 16;
	_in.Tex1 = _in.Tex0;
	_in.Ambient = ambientLighting;
	
	float4 LightColor = 0;
	for(int i = 0;i < 4;i++)
		LightColor += DoLight(_in.Pos,inNormal.xyz,i);
	
	_in.Diffuse = LightColor;
	
	return _in;
}

float4 ps_main(PS_INPUT _in) :	COlOR0
{
	float4 color;
	
	color = lerp(tex2D(tSkin1,_in.Tex0),tex2D(tSkin2,_in.Tex1),tex2D(tBlend,_in.Tex2));
	color *= _in.Ambient + _in.Diffuse;
	
	_in.Tex3 += vecTime.w * 0.001;
	color += lerp(float4(0,0,0,0),fogColor,tex2D(tFogBlend,_in.Tex3));
	
	return color;
}

////////////////////// - TECHNIQUE CODE - /////////////////////
technique multitex_technique
{
	pass p1
	{
		VertexShader = compile vs_2_0 vs_main();
		PixelShader = compile ps_2_0 ps_main();
	}
}
// Fallback Technique:
// - Only 4 dynamic lights (down from 8)
// - No moving fog (it's stationary)
technique multitexFallback_technique
{
	pass p1
	{
		VertexShader = compile vs_1_1 vs_main_FALLBACK();
		PixelShader = compile ps_1_4 ps_main();
	}
}