#include <lights>

texture entSkin1;

sampler tSkin = sampler_state {Texture = <entSkin1>;};

float4x4 matWorldViewProj;
float4 ambientLighting = {0.2,0.2,0.2,0};

////////////////////// - PASS ONE CODE - //////////////////////
struct VS_OUTPUT1
{
	float4 Pos :	POSITION;
};

VS_OUTPUT1 vs_main1(float4 inPos: POSITION,float3 inNormal: NORMAL)
{
	VS_OUTPUT1 Out;
	
	inPos.x += inNormal.x;
	inPos.y += inNormal.y;
	inPos.z += inNormal.z;
	
	Out.Pos = mul(inPos,matWorldViewProj);
	
	return Out;
}

float4 ps_main1() :	COLOR0
{
	float4 color;
	
	color[0] = 0;
	color[1] = 0;
	color[2] = 0;
	color[3] = 1;
	
	return color;
}

////////////////////// - PASS TWO CODE - //////////////////////
struct VS_OUTPUT2
{
	float4 Pos :	POSITION;
	float2 Tex :	TEXCOORD0;
	float4 Light :	COLOR0;
};

VS_OUTPUT2 vs_main2(float4 inPos: POSITION,float2 inTex: TEXCOORD0,float3 inNormal: NORMAL)
{
	VS_OUTPUT2 Out;
	
	Out.Pos = mul(inPos,matWorldViewProj);
	Out.Tex = inTex;
	
	float4 LightColor = 0;
	for(int i = 0;i < 8;i++)
		LightColor += DoLight(inPos,inNormal.xyz,i);
	
	Out.Light = LightColor;
	
	return Out;
}

float4 ps_main2(float2 inTex: TEXCOORD0,float4 inDiffuse: COLOR0) :	COLOR0
{
	float4 color;
	
	color = tex2D(tSkin,inTex);
	
  	color = round(color * 6) / 6.0;
  	
  	color *= round((inDiffuse + ambientLighting) * 4) / 4.0;
	
	return color;
}

////////////////////// - TECHNIQUE CODE - /////////////////////
technique outline_technique
{
	pass p1
	{
		cullmode = cw;
		VertexShader = compile vs_1_1 vs_main1();
		PixelShader = compile ps_2_0 ps_main1();
	}
	
	pass p2
	{
		cullmode = ccw;
		VertexShader = compile vs_2_0 vs_main2();
		PixelShader = compile ps_2_0 ps_main2();
	}
}