///////////////////////////////////////////////////////////
// Shader Showcase - Code for Scene 1
//
// (c) 2009 by Claus N.
// www.nighthawk.dk
///////////////////////////////////////////////////////////
var scene1_movementSpeed = 6;
var scene1_rotateSpeed = 30;
var scene1_runspeedFactor = 1.8;
var Scene1_SunPan = 0;
var Scene1_SunTilt = 0;
var Scene1_SunBrightness = 0;
///////////////////////////////////////////////////////////
BMAP* Scene1_OptionsBG = "Scene1_OptionsBG.tga";
///////////////////////////////////////////////////////////
STRING* Scene1_OptionsSTR = "Adjust sun pan, tilt\nand brightness:";
STRING* Scene1_HelpStr = "Move using W-S-A-D-keys.\nPress and hold Shift for running.\nUse mouse left to drag yellow light.\nUse mouse right to drag blue light.";
STRING* Scene1_InfoStr = "This level is using the default\nshaders from mtlFX.c and mtlView.c.\n\nWalk around and drag the two lights\nwith the mouse to watch the shaders\nin action.";

STRING* Scene1_LightEntStr = "elf.mdl";
///////////////////////////////////////////////////////////
ENTITY* Scene1_MLLightEnt;
ENTITY* Scene1_MRLightEnt;
///////////////////////////////////////////////////////////
PANEL* Scene1_Options =
{
	layer = 1;
	bmap = Scene1_OptionsBG;
	hslider(25,99,272,SliderMap,0,360,Scene1_SunPan);
	hslider(25,126,272,SliderMap,0,360,Scene1_SunTilt);
	hslider(25,153,272,SliderMap,0,100,Scene1_SunBrightness);
}
///////////////////////////////////////////////////////////
TEXT* Scene1_OptionsTXT =
{
	layer = 2;
	string = Scene1_OptionsSTR;
	font = default_font;
	red = 204;
	green = 204;
	blue = 204;
	flags |= (CENTER_X | OUTLINE);
}
///////////////////////////////////////////////////////////
void Scene1_HideOptionsPan()
{
	Scene1_Options.flags &= ~SHOW;
	Scene1_OptionsTXT.flags &= ~SHOW;
}
///////////////////////////////////////////////////////////
void scene1_attachToEntity()
{
	VECTOR* _temp = malloc(sizeof(VECTOR));
	
	my.flags |= PASSABLE;
	
	my.red = 255;
	my.green = 204;
	my.blue = 64;
	my.lightrange = 300;
	
	while(you)
	{
		vec_for_vertex(_temp,my,16);
		_temp.x += random(5) - random(5);
		_temp.y += random(5) - random(5);
		_temp.z += random(4) - random(4);
		
		effect(FireEffect,1,_temp,nullvector);
		
		my.lightrange = clamp(my.lightrange + (random(50) - random(50)) * time_step,250,400);
		
		vec_set(my.x,you.x);
		vec_set(my.pan,you.pan);
		
		my.frame = you.frame;
		my.nextframe = you.nextframe;
		
		wait(1);
	}
	
	free(_temp);
}
///////////////////////////////////////////////////////////
void scene1_npcEvent()
{
	vec_set(my.skill4,my.skill1);
}
///////////////////////////////////////////////////////////
action scene1_npc()
{
	my.ANIM_SPEED_STAND = 2;
	my.ANIM_SPEED_WALK = 4;
	my.ANIM_SPEED_RUN = 10;
	
	my.emask |= (ENABLE_BLOCK | ENABLE_ENTITY);
	my.event = scene1_npcEvent;
	
	vec_set(my.skill1,my.x);
	
	ent_create("mace.mdl",my.x,scene1_attachToEntity);
	
	while(my)
	{
		animate_me();
		
		if(vec_dist(my.skill4,nullvector) == 0)
		{
			my.ANIM_STATE = ANIM_STAND;
			
			if(random(10) > 9.950)
			{
				my.skill4 = my.skill1 + random(200) - random(200);
				my.skill5 = my.skill2 + random(30) - random(30);
				my.skill6 = my.skill3;
			}
		}
		else
		{
			my.ANIM_STATE = ANIM_WALK;
			
			if(vec_dist(my.x,my.skill4) > 20)
			{
				vec_set(target,my.skill4);
				vec_sub(target,my.x);
				vec_to_angle(my.pan,target);
				
				c_move(my,vector(scene1_movementSpeed * time_step,0,0),nullvector,IGNORE_PASSABLE + GLIDE);
			}
			else
			{
				vec_set(my.skill4,nullvector);
			}
		}
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
action scene1_player()
{
	var temp[3];
	
	my.ANIM_SPEED_STAND = 2;
	my.ANIM_SPEED_WALK = 4;
	my.ANIM_SPEED_RUN = 10;
	
	ent_create("mace.mdl",my.x,scene1_attachToEntity);
	
	while(my)
	{
		animate_me();
		
		temp[0] = my.x - camera.x;
		temp[1] = my.y - camera.y;
		temp[2] = my.z - camera.z;
		vec_to_angle(camera.pan,vector(temp[0],temp[1],temp[2]));
		
		my.skill1 = -sign(key_s - key_w*2) * scene1_movementSpeed * time_step;
		my.pan += (key_a - key_d) * scene1_rotateSpeed * time_step;
		
		if(my.skill1 != 0)
		{
			if(key_shift)
			{
				my.skill1 *= scene1_runspeedFactor;
				my.ANIM_STATE = (my.ANIM_STATE != ANIM_SPECIAL) * ANIM_RUN;
			}
			else
			{
				my.ANIM_STATE = (my.ANIM_STATE != ANIM_SPECIAL) * ANIM_WALK;
			}
		}
		else
		{
			my.ANIM_STATE = (my.ANIM_STATE != ANIM_SPECIAL) * ANIM_STAND;
		}
		
		c_move(my,vector(my.skill1,0,0),nullvector,IGNORE_PASSABLE + GLIDE);
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void Scene1_LightEnt()
{
	my.flags |= PASSABLE;
	
	while(my && Scene1_Init == true)
	{
		my.lightrange = clamp(my.lightrange + (random(50) - random(50)) * time_step,250,400);
		
		effect(ParticleEffect1,1,my.x,nullvector);
		
		ent_animate(my,"",my.skill48,ANM_CYCLE);
		my.skill48 += 5 * time_step;
		my.skill48 %= 100;
		
		wait(1);
	}
	
	ptr_remove(my);
}
///////////////////////////////////////////////////////////
void Scene1_DragLight(int _mode)
{
	if(_mode == false) {return;}
	
	if(mouse_pos.x > Scene1_Options.pos_x && mouse_pos.y > Scene1_Options.pos_y
		&& mouse_pos.x < Scene1_Options.pos_x + bmap_width(Scene1_Options.bmap)
		&& mouse_pos.y < Scene1_Options.pos_y + bmap_height(Scene1_Options.bmap)
		&& (Scene1_Options.flags & SHOW) != false)
	{
		return;
	}
	
	if(mouse_pos.x > InfoPan.pos_x && mouse_pos.y > InfoPan.pos_y
		&& mouse_pos.x < InfoPan.pos_x + bmap_width(InfoPan.bmap)
		&& mouse_pos.y < InfoPan.pos_y + bmap_height(InfoPan.bmap)
		&& (InfoPan.flags & SHOW) != false)
	{
		return;
	}
	
	if(mouse_pos.x > MainMenu.pos_x && mouse_pos.y > MainMenu.pos_y
		&& mouse_pos.x < MainMenu.pos_x + bmap_width(MainMenu.bmap)
		&& mouse_pos.y < MainMenu.pos_y + bmap_height(MainMenu.bmap)
		&& MainMenuVisible != false)
	{
		return;
	}
	
	if(mouse_pos.x > VolumePan.pos_x && mouse_pos.y > VolumePan.pos_y
		&& mouse_pos.x < VolumePan.pos_x + bmap_width(VolumePan.bmap)
		&& mouse_pos.y < VolumePan.pos_y + bmap_height(VolumePan.bmap)
		&& (VolumePan.flags & SHOW) != false)
	{
		return;
	}
	
	VECTOR* _temp = malloc(sizeof(VECTOR));
	
	_temp.x = mouse_pos.x;
	_temp.y = mouse_pos.y;
	_temp.z = 0;
	
	vec_set(target,_temp);
	
	target.z = 5000;
	
	vec_for_screen(_temp,camera);
	vec_for_screen(target,camera);
	
	result = c_trace(_temp,target,IGNORE_CONTENT + IGNORE_MODELS);
	
	if(result > 0)
	{
		target.x += normal.x * 20;
		target.y += normal.y * 20;
		target.z += normal.z * 20;
		
		if(_mode > 0)
		{
			if(Scene1_MLLightEnt == null)
			{
				Scene1_MLLightEnt = ent_create(Scene1_LightEntStr,_temp,Scene1_LightEnt);
				Scene1_MLLightEnt.red = 255;
				Scene1_MLLightEnt.green = 255;
				Scene1_MLLightEnt.blue = 0;
			}
			
			vec_set(Scene1_MLLightEnt.x,target);
		}
		else
		{
			if(Scene1_MRLightEnt == null)
			{
				Scene1_MRLightEnt = ent_create(Scene1_LightEntStr,_temp,Scene1_LightEnt);
				Scene1_MRLightEnt.red = 0;
				Scene1_MRLightEnt.green = 0;
				Scene1_MRLightEnt.blue = 255;
			}
			
			vec_set(Scene1_MRLightEnt.x,target);
		}
	}
	
	free(_temp);
}
///////////////////////////////////////////////////////////
void ProcessScene1()
{
	proc_kill(4);
	
	Scene1_MLLightEnt = null;
	Scene1_MRLightEnt = null;
	
	str_cpy((*InfoText.pstring),Scene1_HelpStr);
	
	sky_color.red = 1;
	sky_color.green = 1;
	sky_color.blue = 1;
	
	camera.x = 300;
	camera.y = -300;
	camera.z = 350;
	
	Scene1_SunPan = sun_angle.pan;
	Scene1_SunTilt = sun_angle.tilt;
	Scene1_SunBrightness = 50;
	
	Scene1_Options.pos_x = screen_size.x - bmap_width(Scene1_Options.bmap);
	Scene1_Options.pos_y = screen_size.y - bmap_height(Scene1_Options.bmap);
	Scene1_Options.flags |= SHOW;
	
	Scene1_OptionsTXT.pos_x = Scene1_Options.pos_x + (bmap_width(Scene1_Options.bmap) / 2);
	Scene1_OptionsTXT.pos_y = Scene1_Options.pos_y + 40;
	Scene1_OptionsTXT.flags |= SHOW;
	
	Scene1_Init = true;
	
	musicSetActive(musicScene1,false);
	musicDoLoop(musicScene1,50,200);
	
	while(CurrentScene == SCENE_DEFSHADERS)
	{
		sun_angle.pan = Scene1_SunPan;
		sun_angle.tilt = Scene1_SunTilt;
		sun_light = Scene1_SunBrightness;
		
		if(!fadeMapVisible())
		{
			if(mouse_left) {Scene1_DragLight(1);}
			if(mouse_right) {Scene1_DragLight(-1);}
		}
		
		if(MainMenuVisible != false)
		{
			InfoPan.flags &= ~SHOW;
			InfoText.flags &= ~SHOW;
		}
		else
		{
			InfoPan.flags |= SHOW;
			InfoText.flags |= SHOW;
		}
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void Scene1_InfoPanClicked(var num)
{
	if(num == 1)
	{
		str_cpy((*InfoText.pstring),Scene1_HelpStr);
	}
	if(num == 2)
	{
		str_cpy((*InfoText.pstring),Scene1_InfoStr);
	}
}
///////////////////////////////////////////////////////////
