///////////////////////////////////////////////////////////
// Shader Showcase - Code for Scene 2
//
// (c) 2009 by Claus N.
// www.nighthawk.dk
///////////////////////////////////////////////////////////
var scene2_movementSpeed = 10;
var scene2_rotateSpeed = 70;
var scene2_cameraRotateSpeed = 10;
var scene2_cameraDist = 150;
var scene2_cameraHeight = 50;
var scene2_cameraCollCorrection = 10;
var scene2_cameraAngle;
var scene2_cameraSpeed;
///////////////////////////////////////////////////////////
SOUND* Scene2_disappear = "disappear.wav";
SOUND* Scene2_reappear = "reappear.wav";
///////////////////////////////////////////////////////////
void Scene2_adjustCamera();
///////////////////////////////////////////////////////////
STRING* Scene2_HelpStr = "Move using W-S-A-D-keys.\n\nTouch the the flying things to\nmake them disappear. They\nwill respawn in a while.";
STRING* Scene2_InfoStr = "For the entities in this level I have\nwritten a toon shader, which adds an\noutline and reduces the amount of\ndifferent colors available, to achieve\nNPR (non-photorealistic rendering).\n\nRequires vs 2.0 and ps 2.0";
///////////////////////////////////////////////////////////
MATERIAL* mat_scene2_toon =
{
	effect = "nha_toonshader.fx";
}
///////////////////////////////////////////////////////////
action scene2_tree()
{
	my.flags |= PASSABLE;
	
	my.pan = random(360);
	my.skill1 = 0.75 + random(0.5);
	vec_scale(my.scale_x,my.skill1);
	
	my.material = mat_scene2_toon;
	
	result = c_trace(my.x,vector(my.x,my.y,my.z-500),IGNORE_MODELS);
	
	if(result > 0)
	{
		c_updatehull(me,1);
		my.z -= result + my.min_z;
	}
}
///////////////////////////////////////////////////////////
action scene2_player()
{
	my.material = mat_scene2_toon;
	
	player = me;
	
	scene2_cameraAngle = my.pan;
	
	my.ANIM_SPEED_STAND = 2;
	my.ANIM_SPEED_WALK = 4;
	my.ANIM_SPEED_RUN = 10;
	
	while(my)
	{
		animate_me();
		
		Scene2_adjustCamera();
		
		my.skill1 = -sign(key_s - key_w*2) * scene2_movementSpeed * time_step;
		
		if(key_a | key_d | key_w | key_s)
		{
			my.pan = scene2_cameraAngle - scene2_rotateSpeed * (key_d - key_a);
		}
		
		if(my.skill1 != 0)
		{
			my.ANIM_STATE = (my.ANIM_STATE != ANIM_SPECIAL) * ANIM_RUN;
		}
		else
		{
			my.ANIM_STATE = (my.ANIM_STATE != ANIM_SPECIAL) * ANIM_STAND;
		}
		
		c_move(my,vector(my.skill1,0,0),nullvector,IGNORE_PASSABLE + GLIDE);
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void scene2_flyingLight()
{
	my.material = mat_scene2_toon;
	
	my.red = 255;
	my.green = 255;
	my.blue = 128;
	my.lightrange = 300;
	
	my.flags |= PASSABLE;
	
	my.skill1 = my.z;
	
	my.pan = random(360);
	
	while(my)
	{
		ent_animate(my,"",my.skill48,ANM_CYCLE);
		my.skill48 += 10 * time_step;
		
		if(my.flags & FLAG1)
		{
			my.z -= (0.1 + random(0.8)) * time_step;
			if(my.z < my.skill1 - 2 - random(8)) {my.flags &= ~FLAG1;}
		}
		else
		{
			my.z += (0.1 + random(0.8)) * time_step;
			if(my.z > my.skill1 + 2 + random(8)) {my.flags |= FLAG1;}
		}
		
		effect(FireEffect2,1,my.x,nullvector);
		
		if(player)
		{
			if(vec_dist(my.x,player.x) <= 20)
			{
				my.flags |= INVISIBLE;
				effect(scene2_dissappearEffect,100,my.x,nullvector);
				my.lightrange = 0;
				
				ent_playsound(my,Scene2_disappear,200);
				
				my.skill2 = random(150) + 100;
				while(my.skill2 > 0) {my.skill2 -= 1 * time_step;wait(1);}
				
				effect(scene2_reappearEffect,150,my.x,nullvector);
				my.lightrange = 300;
				
				wait(-0.5);
				ent_playsound(my,Scene2_reappear,200);
				wait(-0.5);
				
				my.flags &= ~INVISIBLE;
			}
		}
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void Scene2_adjustCamera()
{
		var tmpCamPan = scene2_cameraAngle;
		var tmpMyPan = my.pan;
		
		while(tmpCamPan < 0) {tmpCamPan += 360;}
		while(tmpMyPan < 0) {tmpMyPan += 360;}
		
		tmpCamPan %= 360;
		tmpMyPan %= 360;
		
		scene2_cameraSpeed -= 1 * time_step;
		
		if(abs(tmpCamPan - tmpMyPan) < scene2_cameraRotateSpeed || abs(tmpCamPan - tmpMyPan) > 360-scene2_cameraRotateSpeed)
		{
			if(abs(scene2_cameraSpeed) < 0.5 || abs(tmpCamPan - tmpMyPan) < 5 || abs(tmpCamPan - tmpMyPan) > 360-5)
			{
				scene2_cameraSpeed = 0;
				scene2_cameraAngle = my.pan;
			}
			else
			{
				scene2_cameraSpeed -= scene2_cameraSpeed * time_step;
			}
		}
		else
		{
			if(maxv(tmpCamPan,tmpMyPan) - minv(tmpCamPan,tmpMyPan) > 180)
			{
				scene2_cameraSpeed += scene2_cameraRotateSpeed
					* sign((maxv(tmpCamPan,tmpMyPan) == tmpCamPan)
					- (maxv(tmpCamPan,tmpMyPan) == tmpMyPan))
					* time_step;
			}
			else
			{
				scene2_cameraSpeed -= scene2_cameraRotateSpeed
					* sign((maxv(tmpCamPan,tmpMyPan) == tmpCamPan)
					- (maxv(tmpCamPan,tmpMyPan) == tmpMyPan)*2)
					* time_step;
			}
		}
		
		scene2_cameraSpeed = clamp(scene2_cameraSpeed,-scene2_cameraRotateSpeed*4,scene2_cameraRotateSpeed*4);
		scene2_cameraAngle += scene2_cameraSpeed * time_step * 0.25;
		
		target.x = my.x + fcos(scene2_cameraAngle,-scene2_cameraDist);
		target.y = my.y + fsin(scene2_cameraAngle,-scene2_cameraDist);
		target.z = my.z + scene2_cameraHeight;
		
		result = c_trace(my.x,target,IGNORE_ME | IGNORE_MODELS);
		
		if(result > 0)
		{
			target.x = my.x + fcos(scene2_cameraAngle,-result + scene2_cameraCollCorrection);
			target.y = my.y + fsin(scene2_cameraAngle,-result + scene2_cameraCollCorrection);
			target.z = my.z + scene2_cameraHeight;
			
			vec_set(camera.x,target);
		}
		else
		{
			camera.x = my.x + fcos(scene2_cameraAngle,-scene2_cameraDist);
			camera.y = my.y + fsin(scene2_cameraAngle,-scene2_cameraDist);
			camera.z = my.z + scene2_cameraHeight;
		}
		
		vec_set(target,my.x);
		vec_sub(target,camera.x);
		vec_to_angle(camera.pan,target);
}
///////////////////////////////////////////////////////////
void ProcessScene2()
{
	proc_kill(4);
	
	str_cpy((*InfoText.pstring),Scene2_HelpStr);
	
	musicSetActive(musicScene2,false);
	musicDoLoop(musicScene2,50,200);
	
	while(CurrentScene == SCENE_TOON)
	{
		if(MainMenuVisible != false)
		{
			InfoPan.flags &= ~SHOW;
			InfoText.flags &= ~SHOW;
		}
		else
		{
			InfoPan.flags |= SHOW;
			InfoText.flags |= SHOW;
		}
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void Scene2_InfoPanClicked(var num)
{
	if(num == 1)
	{
		str_cpy((*InfoText.pstring),Scene2_HelpStr);
	}
	if(num == 2)
	{
		str_cpy((*InfoText.pstring),Scene2_InfoStr);
	}
}
///////////////////////////////////////////////////////////
