///////////////////////////////////////////////////////////
// Shader Showcase - Code for Scene 3
//
// (c) 2009 by Claus N.
// www.nighthawk.dk
///////////////////////////////////////////////////////////
STRING* Scene3_HelpStr = "Move using W-S-A-D-keys.\n\nPress and hold the right mouse button\nto look around.\n\nPress and hold both mouse buttons\nsimultaneously to move.";
STRING* Scene3_InfoStr = "A small island using a multitexturing\nshader with support for fog and 8\ndynamic lights. The terrain shader\ndoes as well include moving fog on\nthe terrain surface.\n\nTechnique requirements:\nNormal: vs 2.0 and ps 2.0.\nFallback: vs 1.1 and ps 1.4";
///////////////////////////////////////////////////////////
BMAP* Scene3_TerTex1 = "tilegrass2.bmp";
BMAP* Scene3_TerTex2 = "tilesand2.bmp";
BMAP* Scene3_TerTex3 = "terrainfogblendtex.bmp";
MATERIAL* mat_scene3_multitex = 
{
	skin1 = Scene3_TerTex1;
	skin2 = Scene3_TerTex2;
	skin3 = Scene3_TerTex3;
	effect = "nha_terrainMultitex.fx";
}
///////////////////////////////////////////////////////////
// Material to prevent alpha-sorting errors with the trees
MATERIAL* mat_scene3_alphaTest =
{
  effect =
  "
    technique alpha_test
    {
      pass p1
      {
        zWriteEnable = true;
        alphaTestEnable = true;
      }
    }
  ";
}
///////////////////////////////////////////////////////////
action Scene3_island()
{
	my.material = mat_scene3_multitex;
}
///////////////////////////////////////////////////////////
action Scene3_tree()
{
	my.material = mat_scene3_alphaTest;
	
	my.flags |= PASSABLE;
	
	vec_set(target,my.x);
	target.z -= 1000;
	result = c_trace(my.x,target,IGNORE_PASSABLE);
	
	if(result + my.min_z > 0)
	{
		my.z -= result + my.min_z + 5;
	}
}
///////////////////////////////////////////////////////////
action Scene3_player()
{
	my.flags |= INVISIBLE;
	
	while(my)
	{
		if(mouse_right == 0 && mouse_mode == 0)
		{
			setMouse(true);
		}
		else if(mouse_right != 0 && mouse_mode != 0)
		{
			mouse_mode = 0;
		}
		
		my.skill1 = 15 * time_step * -sign(key_s - (key_w | (mouse_left & mouse_right != 0))*2);
		my.skill2 = 15 * time_step * -sign(key_d - key_a*2);
		my.skill3 = 0;
		
		my.pan -= mouse_force.x * 10 * time_step;
		my.tilt = clamp(my.tilt + mouse_force.y * 10 * time_step,-80,80);
		
		camera.pan = my.pan;
		camera.tilt = my.tilt;
		
		c_move(my,my.skill1,nullvector,IGNORE_PASSABLE + GLIDE);
		
		my.x = clamp(my.x,-1500,1500);
		my.y = clamp(my.y,-1500,1500);
		my.z = clamp(my.z,70,1000);
		
		vec_set(camera.x,my.x);
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void ProcessScene3()
{
	proc_kill(4);
	
	str_cpy((*InfoText.pstring),Scene3_HelpStr);
	
	fog_color = 1;
	
	sky_color.red = d3d_fogcolor1.red;
	sky_color.green = d3d_fogcolor1.green;
	sky_color.blue = d3d_fogcolor1.blue;
	
	musicSetActive(musicScene3,false);
	musicDoLoop(musicScene3,50,200);
	
	while(CurrentScene == SCENE_TERRAIN)
	{
		if(MainMenuVisible != false)
		{
			InfoPan.flags &= ~SHOW;
			InfoText.flags &= ~SHOW;
		}
		else
		{
			InfoPan.flags |= SHOW;
			InfoText.flags |= SHOW;
		}
		
		wait(1);
	}
}
///////////////////////////////////////////////////////////
void Scene3_InfoPanClicked(var num)
{
	if(num == 1)
	{
		str_cpy((*InfoText.pstring),Scene3_HelpStr);
	}
	if(num == 2)
	{
		str_cpy((*InfoText.pstring),Scene3_InfoStr);
	}
}
///////////////////////////////////////////////////////////
